#include <cv.h>
#include <highgui.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

void DrawFres(CvArr* img, int x0, int y0, float *Xg, float *Xt);
void CalcFres(float* Xg, float* Xt);
float Pi = 3.1415926;
float gain_range = 100.0;
	float Gain[400];
	float Theta[400];
double  A0, A1, A2, A3, B0, B1, B2, B3;
double  delta = 1.0964782;
double  freq_start = 10;

int main(int argc, char *argv[]) {

  IplImage *img;
  int i;

  img = cvCreateImage (cvSize (640, 360), 8, 3);

  cvSet (img, cvScalarAll (255), 0);

  A0 = 1;
  A1 = 0;
  A2 = 0;
  A3 = 0;

  B0 = 1;
  B1 = 1e-3;
  B2 = 0;
  B3 = 0;

		CalcFres(Gain, Theta);
		DrawFres(img, 100, 60, Gain, Theta);

    while(1) {
        cvNamedWindow ("Freqency Response", CV_WINDOW_AUTOSIZE);
        cvShowImage ("Freqency Response", img);
        if(cvWaitKey(1)>=0) break;
    }

  return 0;
}

void CalcFres(float* Xg, float* Xt) {
        int i, j;
        double omega;
        double frequency;
        double AReal;
        double AImag;
        double AAbs2;
        double AThita;
        double BReal;
        double BImag;
        double BAbs2;
        double BThita;
        double HzdB;
        double HThita;

        for(i = 0; i < 180; i++) {
                
                frequency = freq_start;
                for( j = 0; j < i; j++ ) {
                        frequency = frequency * delta;
                }
                omega = 2 * Pi * frequency;

                AReal = A0 - omega * omega * A2 ;
                AImag = omega * A1 - omega * omega * omega * A3;

                BReal = B0 - omega * omega * B2;
                BImag = omega * B1 - omega * omega * omega * B3;

                AAbs2 = AReal * AReal + AImag * AImag;
                BAbs2 = BReal * BReal + BImag * BImag;

                if( AReal >= 0 ) { 
                        AThita = atan(AImag / AReal);
                } else {
                        AThita = atan(AImag / AReal) + Pi;
                }
                if( BReal >= 0 ) {
                        BThita = atan(BImag / BReal);
                } else {
                        BThita = atan(BImag / BReal) + Pi;
                }

                HzdB = 10 * log10(AAbs2/BAbs2);
                HThita = AThita - BThita;
                if(frequency < 1000000) Xg[i] = HzdB;
		else Xg[i] = -100;
                Xt[i] = HThita;
//		printf("i = %d, f = %f, g = %f\n", i, frequency, Xg[i]);
        }
}

void DrawFres(CvArr* img, int x0, int y0, float *Xg, float *Xt)
{
	float tmp_xg[400], tmp_xt[400];
	int	i, k;
	char freq_char[15];
	char gain_char[15];
	float xstep = 4.0;
	float xscale_shift = 0.0;
	float xscale_start = 0.0;
	float xscale;

  CvFont font;
  int font_face[] = {
    CV_FONT_HERSHEY_SIMPLEX,
    CV_FONT_HERSHEY_PLAIN,
    CV_FONT_HERSHEY_DUPLEX,
    CV_FONT_HERSHEY_COMPLEX,
    CV_FONT_HERSHEY_TRIPLEX,
    CV_FONT_HERSHEY_COMPLEX_SMALL,
    CV_FONT_HERSHEY_SCRIPT_SIMPLEX,
    CV_FONT_HERSHEY_SCRIPT_COMPLEX
  };

  cvInitFont(&font, font_face[0], 0.5, 0.5, 0, 1, 8);

	for (i = 0; i < 400; i++) tmp_xg[i] = 0.0;

	cvPutText(img, "Gain [dB]", cvPoint(x0 - 50, y0 - 28) , &font, CV_RGB(0, 0, 0));
	sprintf(gain_char,"%5.1f", gain_range/5.0);
	cvPutText(img, gain_char, cvPoint(x0 - 50, y0 - 0), &font, CV_RGB(0, 0, 0));

	cvPutText(img, "  0", cvPoint(x0 - 30, y0 + 40), &font, CV_RGB(0, 0, 0));

	sprintf(gain_char,"-%5.1f", gain_range/2.0);
	cvPutText(img, gain_char, cvPoint( x0 - 60, y0 + 145 ), &font, CV_RGB(0, 0, 0));
	sprintf(gain_char,"-%5.1f", gain_range);
	cvPutText(img, gain_char, cvPoint( x0 - 60, y0 + 240 ), &font, CV_RGB(0, 0, 0));
	
	cvPutText(img, "Phase [rad]", cvPoint( x0 + 360 - 10, y0 - 20), &font, CV_RGB(0,0,0));
	cvPutText(img, "  pi ", cvPoint( x0 + 360 + 10, y0 + 60 ), &font, CV_RGB(0,0,0));
	cvPutText(img, " pi/2", cvPoint( x0 + 360 + 10, y0 + 100 ), &font, CV_RGB(0,0,0));
	cvPutText(img, "   0 ", cvPoint( x0 + 360 + 10, y0 + 140 ), &font, CV_RGB(0,0,0));
	cvPutText(img, "-pi/2", cvPoint( x0 + 360 + 10, y0 + 180 ), &font, CV_RGB(0,0,0));
	cvPutText(img, " -pi ", cvPoint( x0 + 360 + 10, y0 + 220 ), &font, CV_RGB(0,0,0));

	sprintf(freq_char,"%12.2f", freq_start);
	cvPutText(img, freq_char, cvPoint( x0 - 80, y0 + 255 ), &font, CV_RGB(0,0,0));
	sprintf(freq_char,"%12.2f", freq_start*10);
	cvPutText(img, freq_char, cvPoint( x0 - 80 + 25*xstep, y0 + 255 ), &font, CV_RGB(0,0,0));
	sprintf(freq_char,"%12.2f", freq_start*100);
	cvPutText(img, freq_char, cvPoint( x0 - 80 + 25*2*xstep, y0 + 255 ), &font, CV_RGB(0,0,0));
	sprintf(freq_char,"%12.2f", freq_start*1000);
	cvPutText(img, freq_char, cvPoint( x0 - 80 + 25*3*xstep, y0 + 255 ), &font, CV_RGB(0,0,0));
	cvPutText(img, "[Hz]", cvPoint( x0 - 10 + 410, y0 + 255 ), &font, CV_RGB(0,0,0));


	for(i = 0; i <= 12; i++) {
		cvLine( img, cvPoint(x0      , y0 + i * 20),
				cvPoint(x0 + 360, y0 + i * 20),
				CV_RGB(200, 200, 200), 1, 8, 0);
	}

	for(k = 1; k <= 9; k++) {
		for(i = 0; i <= 5; i++) {
			if(k == 1) xscale_shift = 0.0;
			if(k == 2) xscale_shift = 7.5;
			if(k == 3) xscale_shift = 11.9;
			if(k == 4) xscale_shift = 15.1;
			if(k == 5) xscale_shift = 17.5;
			if(k == 6) xscale_shift = 19.5;
			if(k == 7) xscale_shift = 21.1;
			if(k == 8) xscale_shift = 22.6;
			if(k == 9) xscale_shift = 23.9;
			xscale = i * 25 * xstep + xscale_shift * xstep;
			if(xscale < 360) {
				cvLine( img, cvPoint(x0 + xscale, y0),
					cvPoint(x0 + xscale, y0+240),
					CV_RGB(200, 200, 200), 1, 8, 0);
			}
		}
	}

	// Frame drawing
	for(i = 0; i <= 12; i = i + 12) {
		cvLine( img, cvPoint(x0      , y0 + i * 20),
				cvPoint(x0 + 360, y0 + i * 20),
				CV_RGB(100, 100, 100), 1, 8, 0);
	}
	for(i = 0; i <= 10; i = i + 10) {
		cvLine( img, cvPoint(x0 + i * (360 / 10), y0),
				cvPoint(x0 + i * (360 / 10), y0 + 240),
				CV_RGB(100, 100, 100), 1, 8, 0);
	}

	
	for(i = 0; i < 180; i++) {
    		tmp_xt[i] = 25.5 * Xt[i];
	}

	for(i = 0; i < 179; i++) {
		if( (i+1)*xstep < 360 ) {
			cvLine( img, cvPoint(x0 + (int)(i*xstep), y0 + 140 - (int)(tmp_xt[i])),
					cvPoint(x0 + (int)((i+1)*xstep), y0 + 140 - (int)(tmp_xt[i + 1])),
					CV_RGB(0, 200, 0), 1, 8, 0);
		}
	}
	
	cvPutText(img, "Phase", cvPoint( x0 + 200, y0 - 25 ), &font, CV_RGB(0,0,0));
	cvLine( img, cvPoint(x0 + 250, y0 - 30),
			cvPoint(x0 + 270, y0 - 30),
			CV_RGB(0, 255, 0), 1, 8, 0);

	float ymax, ymin, yrange;

	ymax = 40.0;
	ymin = -200.0;
	yrange = 2.0 * 100.0 / gain_range;

	for(i = 0; i < 180; i++) {
    	tmp_xg[i] = yrange * Xg[i];
		if( tmp_xg[i] < ymin ) tmp_xg[i] = ymin;
		if( tmp_xg[i] > ymax ) tmp_xg[i] = ymax;
	}

	for(i = 0; i < 179; i++) {
		//tmp_xg[i] = 10;
		if( (i+1)*xstep < 360 ) {
			cvLine( img, cvPoint(x0 + (int)(i*xstep), y0 + 40 - (int)(tmp_xg[i])),
					cvPoint(x0 + (int)((i+1)*xstep), y0 + 40 - (int)(tmp_xg[i + 1])),
					CV_RGB(255, 0, 0), 1, 8, 0);

		}
	}
	cvPutText(img, "Gain", cvPoint( x0 + 100, y0 - 25), &font, CV_RGB(0,0,0));
	cvLine( img, cvPoint(x0 + 140, y0 - 30),
			cvPoint(x0 + 160, y0 - 30),
			CV_RGB(255, 0, 0), 1, 8, 0);

}

