#include <cv.h>
#include <highgui.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

#define MAXTAP 1000
#define SCALEX 640
#define SCALEY 320
#define OFTX 40
#define OFTY 20
#define STEPX 4

void DrawFres(CvArr* img, int x0, int y0, float *Xg, float *Xt);
void CalcFres(float* Xg, float* Xt);
void DrawScale(CvArr* img);
void PutParam();
void DoIdft();
void Kaiser(int n, float alpha);
void TxtSave(char *fn, float *iW);

float Pi = 3.1415926;
	float hh[400];
	float Gain[400];
	float Theta[400];
	float Fsamp;
	float gain_range;
	float freq_range;
	int TapN;
	int TapN_2;
	int FresN;
	float lpf_fc1;
	float hpf_fc1;
	float bpf_fc1;
	float bpf_fc2;
	float brf_fc1;
	float brf_fc2;
	float alpha;
	int FilType;
	float FresR[400];
	float FresI[400];
int evenTap;

int main(int argc, char *argv[]) {

  IplImage *img;
  int val, valp1;
  int i;

	Fsamp = 22050;
	gain_range = 100.0;
	freq_range = 10000;//Fsamp/2.0;
	FresN = 400;
	lpf_fc1 = 3000;
	hpf_fc1 = 3000;
	bpf_fc1 = 3000;
	bpf_fc2 = 7000;
	brf_fc1 = 3000;
	brf_fc2 = 7000;
	alpha = 7.0;
	FilType = 0;
	TapN = 140;

  img = cvCreateImage (cvSize (640, 360), 8, 3);

  cvSet (img, cvScalarAll (255), 0);

  CvScalar color = CV_RGB(255, 0, 0);

	if(TapN % 2 == 0) {
		TapN -= 1;
		evenTap = 1;
	} else {
		evenTap = 0;
	}
	TapN_2 = TapN/2;

		PutParam();
		DoIdft();
		Kaiser(TapN, alpha);

		CalcFres(Gain, Theta);
		DrawFres(img, 100, 60, Gain, Theta);

    while(1) {
        cvNamedWindow ("Frequency Response", CV_WINDOW_AUTOSIZE);
        cvShowImage ("Frequency Response", img);
        if(cvWaitKey(1)>=0) break;
    }

        TxtSave("coeff1.txt", hh);

  return 0;
}

void CalcFres(float* Xg, float* Xt) {
	int i, m;
	float	omega;
	float	AReal;
	float	AImag;
	float	AAbs_2;
	float	ATheta;
	float	HdB;

	for(i = 0; i < 360; i++) {
		omega = i * Pi / 180.0 + 0.001;

		AReal = 0; AImag = 0;
		
		for(m = 0; m < TapN; m++) {
			AReal = AReal + hh[m] * cos(m * omega);
			AImag = AImag - hh[m] * sin(m * omega);
		}

		AAbs_2 = AReal * AReal + AImag * AImag;
		ATheta = atan(AImag / AReal);

		HdB = 10 * log10(AAbs_2);

		Xg[i] = HdB;
		Xt[i] = ATheta;
//		printf("%f\n", Xg[i]);
	}
}

void DrawFres(CvArr* img, int x0, int y0, float *Xg, float *Xt)
{

	float start_fr, end_fr, mid_fr;
	float tmp_xg[400], tmp_xt[400];
	int	i;
	char freq_char[15];
	char gain_char[15];
	float xstep = 2.0;

  CvFont font;
  int font_face[] = {
    CV_FONT_HERSHEY_SIMPLEX,
    CV_FONT_HERSHEY_PLAIN,
    CV_FONT_HERSHEY_DUPLEX,
    CV_FONT_HERSHEY_COMPLEX,
    CV_FONT_HERSHEY_TRIPLEX,
    CV_FONT_HERSHEY_COMPLEX_SMALL,
    CV_FONT_HERSHEY_SCRIPT_SIMPLEX,
    CV_FONT_HERSHEY_SCRIPT_COMPLEX
  };

  cvInitFont(&font, font_face[0], 0.5, 0.5, 0, 1, 8);

	for (i = 0; i < 400; i++) tmp_xg[i] = 0.0;

	cvPutText(img, "Gain [dB]", cvPoint(x0 - 50, y0 - 28) , &font, CV_RGB(0, 0, 0));
	sprintf(gain_char,"%5.1f", gain_range/5.0);
	cvPutText(img, gain_char, cvPoint(x0 - 50, y0 - 0), &font, CV_RGB(0, 0, 0));

	cvPutText(img, "  0", cvPoint(x0 - 30, y0 + 40), &font, CV_RGB(0, 0, 0));

	sprintf(gain_char,"-%5.1f", gain_range/2.0);
	cvPutText(img, gain_char, cvPoint( x0 - 60, y0 + 145 ), &font, CV_RGB(0, 0, 0));
	sprintf(gain_char,"-%5.1f", gain_range);
	cvPutText(img, gain_char, cvPoint( x0 - 60, y0 + 240 ), &font, CV_RGB(0, 0, 0));
	
	cvPutText(img, "Phase [rad]", cvPoint( x0 + 360 - 10, y0 - 20), &font, CV_RGB(0,0,0));
	cvPutText(img, "  pi ", cvPoint( x0 + 360 + 10, y0 + 60 ), &font, CV_RGB(0,0,0));
	cvPutText(img, " pi/2", cvPoint( x0 + 360 + 10, y0 + 100 ), &font, CV_RGB(0,0,0));
	cvPutText(img, "   0 ", cvPoint( x0 + 360 + 10, y0 + 140 ), &font, CV_RGB(0,0,0));
	cvPutText(img, "-pi/2", cvPoint( x0 + 360 + 10, y0 + 180 ), &font, CV_RGB(0,0,0));
	cvPutText(img, " -pi ", cvPoint( x0 + 360 + 10, y0 + 220 ), &font, CV_RGB(0,0,0));

	start_fr = 0.0;
	end_fr = start_fr + freq_range / 1.0;
	mid_fr = start_fr + freq_range / 2.0;

	sprintf(freq_char,"%8.0f", start_fr);
	cvPutText(img, freq_char, cvPoint( x0 - 50, y0 + 255 ), &font, CV_RGB(0,0,0));
	sprintf(freq_char,"%8.0f", mid_fr);
	cvPutText(img, freq_char, cvPoint( x0 - 20 + 150, y0 + 255 ), &font, CV_RGB(0,0,0));
	sprintf(freq_char,"%8.0f", end_fr);
	cvPutText(img, freq_char, cvPoint( x0 - 20 + 330, y0 + 255 ), &font, CV_RGB(0,0,0));
	cvPutText(img, "[Hz]", cvPoint( x0 - 10 + 410, y0 + 255 ), &font, CV_RGB(0,0,0));

//	sprintf(freq_char,"hh = %10.3f", hh[0]);
//	cvPutText(img, freq_char, cvPoint( x0 - 50, y0 + 255 ), &font, CV_RGB(0,0,0));

	for(i = 0; i <= 12; i++) {
		cvLine( img, cvPoint(x0      , y0 + i * 20),
				cvPoint(x0 + 360, y0 + i * 20),
				CV_RGB(200, 200, 200), 1, 8, 0);
	}

	for(i = 0; i <= 10; i++) {
		cvLine( img, cvPoint(x0 + i * (360 / 10), y0),
				cvPoint(x0 + i * (360 / 10), y0+240),
				CV_RGB(200, 200, 200), 1, 8, 0);
	}

//	cvLine( img, cvPoint(x0      , y0 + 2 * 20),
//			cvPoint(x0 + 360, y0 + 2 * 20),
//			CV_RGB(100, 100, 100), 1, 8, 0);
//
//	cvLine( img, cvPoint(x0 + 5 * (360 / 10), y0),
//			cvPoint(x0 + 5 * (360 / 10), y0 + 240),
//			CV_RGB(100, 100, 100), 1, 8, 0);

	// Frame drawing
	for(i = 0; i <= 12; i = i + 12) {
		cvLine( img, cvPoint(x0      , y0 + i * 20),
				cvPoint(x0 + 360, y0 + i * 20),
				CV_RGB(100, 100, 100), 1, 8, 0);
	}
	for(i = 0; i <= 10; i = i + 10) {
		cvLine( img, cvPoint(x0 + i * (360 / 10), y0),
				cvPoint(x0 + i * (360 / 10), y0 + 240),
				CV_RGB(100, 100, 100), 1, 8, 0);
	}

	
	for(i = 0; i < 180; i++) {
    		tmp_xt[i] = 25.5 * Xt[i];
	}

	xstep = Fsamp/freq_range;

	for(i = 0; i < 179; i++) {
		if( (i+1)*xstep < 360 ) {
			cvLine( img, cvPoint(x0 + (int)(i*xstep), y0 + 140 - (int)(tmp_xt[i])),
					cvPoint(x0 + (int)((i+1)*xstep), y0 + 140 - (int)(tmp_xt[i + 1])),
					CV_RGB(0, 200, 0), 1, 8, 0);
		}
	}
	
	cvPutText(img, "Phase", cvPoint( x0 + 200, y0 - 25 ), &font, CV_RGB(0,0,0));
	cvLine( img, cvPoint(x0 + 250, y0 - 30),
			cvPoint(x0 + 270, y0 - 30),
			CV_RGB(0, 255, 0), 1, 8, 0);

	float ymax, ymin, yrange;

	ymax = 40.0;
	ymin = -200.0;
	yrange = 2.0 * 100.0 / gain_range;

	for(i = 0; i < 180; i++) {
    	tmp_xg[i] = yrange * Xg[i];
		if( tmp_xg[i] < ymin ) tmp_xg[i] = ymin;
		if( tmp_xg[i] > ymax ) tmp_xg[i] = ymax;
	}

	for(i = 0; i < 179; i++) {
		//tmp_xg[i] = 10;
		if( (i+1)*xstep < 360 ) {
			cvLine( img, cvPoint(x0 + (int)(i*xstep), y0 + 40 - (int)(tmp_xg[i])),
					cvPoint(x0 + (int)((i+1)*xstep), y0 + 40 - (int)(tmp_xg[i + 1])),
					CV_RGB(255, 0, 0), 1, 8, 0);

		}
	}
	cvPutText(img, "Gain", cvPoint( x0 + 100, y0 - 25), &font, CV_RGB(0,0,0));
	cvLine( img, cvPoint(x0 + 140, y0 - 30),
			cvPoint(x0 + 160, y0 - 30),
			CV_RGB(255, 0, 0), 1, 8, 0);

}

void PutParam() {
	int i;
	int fc1int, fc2int;
	float Reduct = -200.0;
	float gain_fc1, gain_fc2, gain_fc3;

	if(FilType == 0) {
		fc1int = (int)(FresN * lpf_fc1 / Fsamp);
		fc2int = fc1int;
		gain_fc1 = pow(10.0, 0.0 / 20.0);
		gain_fc2 = pow(10.0, Reduct / 20.0);
		gain_fc3 = pow(10.0, Reduct / 20.0);
	} else if(FilType == 1) {
		fc1int = (int)(FresN * hpf_fc1 / Fsamp);
		fc2int = fc1int;
		gain_fc1 = pow(10.0, Reduct / 20.0);
		gain_fc2 = pow(10.0, 0.0 / 20.0);
		gain_fc3 = pow(10.0, 0.0 / 20.0);
	} else if(FilType == 2) {
		fc1int = (int)(FresN * bpf_fc1 / Fsamp);
		fc2int = (int)(FresN * bpf_fc2 / Fsamp);
		gain_fc1 = pow(10.0, Reduct / 20.0);
		gain_fc2 = pow(10.0, 0.0 / 20.0);
		gain_fc3 = pow(10.0, Reduct / 20.0);
	} else if(FilType == 3) {
		fc1int = (int)(FresN * brf_fc1 / Fsamp);
		fc2int = (int)(FresN * brf_fc2 / Fsamp);
		gain_fc1 = pow(10.0, 0.0 / 20.0);
		gain_fc2 = pow(10.0, Reduct / 20.0);
		gain_fc3 = pow(10.0, 0.0 / 20.0);
	}

	for (i = 0; i < fc1int; i++) {
		FresR[i]=gain_fc1;
	}
	for (i = fc1int; i < fc2int; i++) {
		FresR[i]=gain_fc2;
	}
	for (i = fc2int; i < FresN - fc2int; i++) {
		FresR[i]=gain_fc3;
	}
	for (i = FresN - fc2int; i < FresN - fc1int; i++) {
		FresR[i]=gain_fc2;
	}
	for (i = FresN - fc1int; i < FresN; i++) {
		FresR[i]=gain_fc1;
	}
	for (i = 0; i < FresN; i++) { // 虚数部はゼロ
		FresI[i]=0.0;
	}
}

void DoIdft() {
	int i,k;
	float omega;
	float cor[400], coi[400];

	for (i = 0; i < FresN; i++) {
		cor[i] = 0.0; coi[i] = 0.0; 
		for (k = 0; k < FresN; k++) {
			omega = 2.0 * Pi * k * i / FresN; 

			cor[i] += FresR[k] * cos(omega) - FresI[k] * sin(omega); 

			coi[i] += FresI[k] * cos(omega) + FresR[k] * sin(omega); 
		}
	}
	for (i = 0; i < FresN; i++) {
		cor[i] /= FresN; coi[i] /= FresN;
	}
	for (i = 0; i < 400; i++) hh[i] = 0;
	for (i = 0; i < TapN_2; i++) {
		hh[i] = cor[TapN_2-i];
	}
	for (i = TapN_2; i < 2*TapN_2+1; i++) {
		hh[i] = cor[i-TapN_2];
	}
}

void Kaiser(int n, float alpha) {
	float win[400];
	float ERR = 1e-14;
	int i;
	float old, diff, p, r, t;
	float i1, i2;
	float a,g;
                                          
	old = p = 0.0;
	i2 = r = diff = 1.0;
	t = (float)(alpha / 2);
	while(diff < -ERR || diff > ERR) {
		p += 1.0;
		r *= (t/p);
		i2 += r*r;
		diff = old - i2;
		old = i2;
	}
	a = (float)(n - 1) / 2;
	for(i = 0; i < n; i++) {
		g = ((float)i - a) / a;
		t = (float)(alpha * sqrt(1.0 - g*g));
		old = p = 0.0;
		i1 = r = diff = 1.0;
		t /= 2;
		while(diff < -ERR || diff > ERR) {
			p += 1.0;
			r *= (t/p);
			i1 += r*r;
			diff = old - i1;
			old = i1;
		}
		win[i] = i1 / i2;
	}
	for(i = 0; i < n; i++) hh[i] *= win[i];
}

void TxtSave(char *fn, float *iW)
{
  int i;
  char charline[40];
  FILE *fp;

  fp = fopen( fn, "w" );
  if( fp == NULL ){
    printf( "%s cannot be opened.\n", fn );
    exit(1);
  }

  fseek( fp, 0L, SEEK_SET );
  for( i = 0; i < TapN; i++) {
    sprintf(charline, "%f\n", iW[i]);
    fputs(charline, fp);
//    printf( "%d\n", iW[i] );
  }
  if( evenTap == 1 ) {
    sprintf(charline, "%f\n", 0.0);
    fputs(charline, fp);
  }

  fclose( fp );
}

