#include <cv.h>
#include <highgui.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

#define MAXTAP 1000
#define SCALEX 640
#define SCALEY 320
#define OFTX 40
#define OFTY 20
#define STEPX 4

void DrawFres(CvArr* img, int x0, int y0, float *Xg, float *Xt);
void CalcFres(float* Xg, float* Xt);
void DrawScale(CvArr* img);
void TxtLoad(char *fn, float *iW);
float MAG = 2.0;
float DIV = 10000.0;
int TapN = 10;
float Pi = 3.1415926;
float gain_range = 100.0;
float	freq_range = 10000;
float	Fsamp = 22050;
	float hh[400];
	float Gain[400];
	float Theta[400];

int main(int argc, char *argv[]) {

  char input_file[256];
  IplImage *img;
  int val, valp1;
  int i;

  img = cvCreateImage (cvSize (640, 360), 8, 3);

  cvSet (img, cvScalarAll (255), 0);

  if (argc < 2) {
      fprintf(stderr, "1st arg must be input file name.\n");
      return -1;
  }

  strcpy(input_file, argv[1]);
  TxtLoad(input_file, hh);

  CvScalar color = CV_RGB(255, 0, 0);

		CalcFres(Gain, Theta);
		DrawFres(img, 100, 60, Gain, Theta);

    while(1) {
        cvNamedWindow ("Frequency Response", CV_WINDOW_AUTOSIZE);
        cvShowImage ("Frequency Response", img);
        if(cvWaitKey(1)>=0) break;
    }

  return 0;
}

void TxtLoad(char *fn, float *iW)
{
  int count = 0;
  char charline[40];
  FILE *fp;

  fp = fopen( fn, "r" );
  if( fp == NULL ){
    printf( "%s cannot be opened.\n", fn );
    exit(1);
  } 

  fseek( fp, 0L, SEEK_SET );
  while( fgets(charline, 40, fp) != NULL ) {
    iW[count] = atof(charline);
    count++;

    if(count == MAXTAP) {
      break;
    }
  } 
  TapN = count;
  printf( "TapN = %d\n", TapN );
  fclose( fp );
}

void CalcFres(float* Xg, float* Xt) {
	int i, m;
	float	omega;
	float	AReal;
	float	AImag;
	float	AAbs_2;
	float	ATheta;
	float	HdB;

	for(i = 0; i < 360; i++) {
		omega = i * Pi / 180.0 + 0.001;

		AReal = 0; AImag = 0;
		
		for(m = 0; m < TapN; m++) {
			AReal = AReal + hh[m] * cos(m * omega);
			AImag = AImag - hh[m] * sin(m * omega);
		}

		AAbs_2 = AReal * AReal + AImag * AImag;
		ATheta = atan(AImag / AReal);

		HdB = 10 * log10(AAbs_2);

		Xg[i] = HdB;
		Xt[i] = ATheta;
	}
}

void DrawFres(CvArr* img, int x0, int y0, float *Xg, float *Xt)
{
	float start_fr, end_fr, mid_fr;
	float tmp_xg[400], tmp_xt[400];
	int	i;
	char freq_char[15];
	char gain_char[15];
	float xstep = 2.0;

  CvFont font;
  int font_face[] = {
    CV_FONT_HERSHEY_SIMPLEX,
    CV_FONT_HERSHEY_PLAIN,
    CV_FONT_HERSHEY_DUPLEX,
    CV_FONT_HERSHEY_COMPLEX,
    CV_FONT_HERSHEY_TRIPLEX,
    CV_FONT_HERSHEY_COMPLEX_SMALL,
    CV_FONT_HERSHEY_SCRIPT_SIMPLEX,
    CV_FONT_HERSHEY_SCRIPT_COMPLEX
  };

  cvInitFont(&font, font_face[0], 0.5, 0.5, 0, 1, 8);

	for (i = 0; i < 400; i++) tmp_xg[i] = 0.0;

	cvPutText(img, "Gain [dB]", cvPoint(x0 - 50, y0 - 28) , &font, CV_RGB(0, 0, 0));
	sprintf(gain_char,"%5.1f", gain_range/5.0);
	cvPutText(img, gain_char, cvPoint(x0 - 50, y0 - 0), &font, CV_RGB(0, 0, 0));

	cvPutText(img, "  0", cvPoint(x0 - 30, y0 + 40), &font, CV_RGB(0, 0, 0));

	sprintf(gain_char,"-%5.1f", gain_range/2.0);
	cvPutText(img, gain_char, cvPoint( x0 - 60, y0 + 145 ), &font, CV_RGB(0, 0, 0));
	sprintf(gain_char,"-%5.1f", gain_range);
	cvPutText(img, gain_char, cvPoint( x0 - 60, y0 + 240 ), &font, CV_RGB(0, 0, 0));
	
	cvPutText(img, "Phase [rad]", cvPoint( x0 + 360 - 10, y0 - 20), &font, CV_RGB(0,0,0));
	cvPutText(img, "  pi ", cvPoint( x0 + 360 + 10, y0 + 60 ), &font, CV_RGB(0,0,0));
	cvPutText(img, " pi/2", cvPoint( x0 + 360 + 10, y0 + 100 ), &font, CV_RGB(0,0,0));
	cvPutText(img, "   0 ", cvPoint( x0 + 360 + 10, y0 + 140 ), &font, CV_RGB(0,0,0));
	cvPutText(img, "-pi/2", cvPoint( x0 + 360 + 10, y0 + 180 ), &font, CV_RGB(0,0,0));
	cvPutText(img, " -pi ", cvPoint( x0 + 360 + 10, y0 + 220 ), &font, CV_RGB(0,0,0));

	start_fr = 0.0;
	end_fr = start_fr + freq_range / 1.0;
	mid_fr = start_fr + freq_range / 2.0;

	sprintf(freq_char,"%10.0f", start_fr);
	cvPutText(img, freq_char, cvPoint( x0 - 70, y0 + 255 ), &font, CV_RGB(0,0,0));
	sprintf(freq_char,"%10.0f", mid_fr);
	cvPutText(img, freq_char, cvPoint( x0 - 30 + 150, y0 + 255 ), &font, CV_RGB(0,0,0));
	sprintf(freq_char,"%10.0f", end_fr);
	cvPutText(img, freq_char, cvPoint( x0 - 30 + 330, y0 + 255 ), &font, CV_RGB(0,0,0));
	cvPutText(img, "[Hz]", cvPoint( x0 - 10 + 410, y0 + 255 ), &font, CV_RGB(0,0,0));

//	sprintf(freq_char,"hh = %10.3f", hh[0]);
//	cvPutText(img, freq_char, cvPoint( x0 - 50, y0 + 255 ), &font, CV_RGB(0,0,0));

	for(i = 0; i <= 12; i++) {
		cvLine( img, cvPoint(x0      , y0 + i * 20),
				cvPoint(x0 + 360, y0 + i * 20),
				CV_RGB(200, 200, 200), 1, 8, 0);
	}

	for(i = 0; i <= 10; i++) {
		cvLine( img, cvPoint(x0 + i * (360 / 10), y0),
				cvPoint(x0 + i * (360 / 10), y0+240),
				CV_RGB(200, 200, 200), 1, 8, 0);
	}

	// Frame drawing
	for(i = 0; i <= 12; i = i + 12) {
		cvLine( img, cvPoint(x0      , y0 + i * 20),
				cvPoint(x0 + 360, y0 + i * 20),
				CV_RGB(100, 100, 100), 1, 8, 0);
	}
	for(i = 0; i <= 10; i = i + 10) {
		cvLine( img, cvPoint(x0 + i * (360 / 10), y0),
				cvPoint(x0 + i * (360 / 10), y0 + 240),
				CV_RGB(100, 100, 100), 1, 8, 0);
	}

	
	for(i = 0; i < 180; i++) {
    		tmp_xt[i] = 25.5 * Xt[i];
	}

	xstep = Fsamp/freq_range;

	for(i = 0; i < 179; i++) {
		if( (i+1)*xstep < 360 ) {
			cvLine( img, cvPoint(x0 + (int)(i*xstep), y0 + 140 - (int)(tmp_xt[i])),
					cvPoint(x0 + (int)((i+1)*xstep), y0 + 140 - (int)(tmp_xt[i + 1])),
					CV_RGB(0, 200, 0), 1, 8, 0);
		}
	}
	
	cvPutText(img, "Phase", cvPoint( x0 + 200, y0 - 30 ), &font, CV_RGB(0,0,0));
	cvLine( img, cvPoint(x0 + 250, y0 - 30),
			cvPoint(x0 + 270, y0 - 30),
			CV_RGB(0, 255, 0), 1, 8, 0);

	float ymax, ymin, yrange;

	ymax = 40.0;
	ymin = -200.0;
	yrange = 2.0 * 100.0 / gain_range;

	for(i = 0; i < 180; i++) {
    	tmp_xg[i] = yrange * Xg[i];
		if( tmp_xg[i] < ymin ) tmp_xg[i] = ymin;
		if( tmp_xg[i] > ymax ) tmp_xg[i] = ymax;
	}

	for(i = 0; i < 179; i++) {
		//tmp_xg[i] = 10;
		if( (i+1)*xstep < 360 ) {
			cvLine( img, cvPoint(x0 + (int)(i*xstep), y0 + 40 - (int)(tmp_xg[i])),
					cvPoint(x0 + (int)((i+1)*xstep), y0 + 40 - (int)(tmp_xg[i + 1])),
					CV_RGB(255, 0, 0), 1, 8, 0);

		}
	}
	cvPutText(img, "Gain", cvPoint( x0 + 100, y0 - 30), &font, CV_RGB(0,0,0));
	cvLine( img, cvPoint(x0 + 140, y0 - 30),
			cvPoint(x0 + 160, y0 - 30),
			CV_RGB(256, 0, 0), 1, 8, 0);

}

